/*

   Author:    Gerhard Rhner
   E-Mail:    GRoehner@t-online.de

   Zweck      : Erzeugt fr eine Anfrage eine Verfolgungsspur.
                Die CALL-Ports werden entsprechend der Aufruf-
                tiefe eingerckt angezeigt.
   Autor      : Gerhard Rhner
   Literatur  : N. Fuchs: Kurs in logischer Programmierung
   Version    : 1.0, 16.11.91, Urfassung
                1.1, 05.12.91, Bercksichtigung von not
                2.0, 13.01.92, Bercksichtigung von Cut
                2.1, 03.02.92, Bercksichtigung von ;
                2.2, 14.12.95, Version fr Swing-Prolog
                2.3, 03.01.96, Library-Version
                
   In das Library-Verzeichnis kopieren, in Prolog ffen und speichern
   und dann "make." aufrufen.
   
*/

:- module(spur, [spur/1]).

spur(Ziel):-
  spur(Ziel, 0, Ergebnis),
  (Ergebnis == fail, !, fail; true).

spur(true, _Tiefe, true):-                          /* facts  */
  !.

spur(!, Tiefe, true):-                              /* Cut     */
  writeline(!, Tiefe).
spur(!, Tiefe, fail):-
  !,
  writeline('Cut!', Tiefe).

spur((A, B), Tiefe, Ergebnis):-                     /* rules  */
  !,
  spur(A, Tiefe, ErgebnisA), 
  (ErgebnisA  == true, spur(B, Tiefe, Ergebnis); 
   ErgebnisA \== true, Ergebnis = ErgebnisA).
   
spur((A; B), Tiefe, Ergebnis):-
  !,
  (spur(A, Tiefe, Ergebnis);
  spur(B, Tiefe, Ergebnis)).

spur(A, Tiefe, true):- 
  system(A), !,
  call(A), 
  writeline(A, Tiefe).
                       
spur(not(A), Tiefe, true):-
  !,
  writeline(not, Tiefe),
  not(spur(A, Tiefe, _)).

spur(A, Tiefe, Ergebnis):- 
  clause(A, B),
  writeline(A, Tiefe), 
  Tiefe1 is Tiefe + 3,
  spur(B, Tiefe1, Ergebnis),
  (Ergebnis == fail, !, fail; true).
                       
spur(A, Tiefe, true):-               /* keine weitere Regel fr A */
  not(clause(A, _)),
  tab(Tiefe), write(A), write('    nein'), nl,
  fail.

writeline(A, Tiefe):-
  tab(Tiefe), write(A), nl.

system(X):- 
  predicate_property(X, built_in).



